import sys
import os
import logging as log
import getopt
import ConfigParser
import ldap
import locale
from datetime import datetime
from smtplib import SMTPException

import ldap_notify.config
import ldap_notify.connection
from ldap_notify import ConfigError
import ldap_notify.globals as g

# add virtualenv path to PYTHON_PATH, so libs are found
virtualenv_path = os.path.dirname(__file__) + '/env'
if virtualenv_path not in sys.path:
	sys.path.insert(0, virtualenv_path)

class SysExitException(Exception):
	def __init__(self, rc):
		super(SysExitException, self).__init__('sys.exit(%i)' % rc)
		self.rc = rc

def usage(): 
	print """Usage: ldap-notify [OPTION]... -c <config_file.conf>

Parameters:'
  -h, --help                    show this help
  -c, --conf                    mandatory parameter: the config file name
  -k                            ignore SSL/TLS certificates
  --dry                         do not send emails or modify anything in ldap
  --test <test-address>         send all mails to the given address
  --restrict <DN>;<CN>;...      restrict notifications to these semicolon 
                                separated user DNs or CNs
  --time <timestamp>            simulate current UTC time (format: 20141031162633Z)
  -v, --verbose                 verbose logging
  -d, --debug                   debug logging
  --print-conf                  print the used configuration to console and exit
"""

def run(config):
	# start the algorithm
	con = ldap_notify.connection.connect_to_ldap(config)
	import ldap_notify.algorithm as algorithm
	if g.DEBUG > 5:
		print repr(algorithm.search_users(config, con))
	
	# process the rules, starting with the smallest interval
	users_without_email = []
	failed_users = []
	notified_users = []
	for rule in config.rules:
		rule_users = algorithm.users_for_rule(config, con, rule)
		
		# assign rule
		for user in rule_users:
			setattr(user, 'rule', rule)
		
		# notify those which have an email
		successful, failed = algorithm.notify_users(config, con, filter(lambda u: u.mail, rule_users), rule)
		notified_users.extend(successful)
		failed_users.extend(failed)
		
		# collect all others for the admin
		users_without_email.extend(filter(lambda u: not u.mail, rule_users))

	# find users without graceLogins
	users_without_grace_logins = algorithm.search_users_without_grace_logins(config, con)

	# send admin email
	algorithm.notify_admin(config, con, notified_users, failed_users, users_without_email, users_without_grace_logins)

def main(argv):
	try:
		locale.setlocale(locale.LC_ALL, "")
	
		# default values
		ignore_cert = None
		config_file = None
		debug = 0
		verbose = False
		now = datetime.utcnow()
		dry = None
		test = None
		test_address = None
		restrict_to_users = None
		print_conf = None
		config_file = None
	
		# parse arguments	
		try:
			opts, args = getopt.getopt(argv, "hc:dvk", ["help", "config=", "time=", "test=", 'dry', "debug", "verbose", "restrict=", "print-conf"])
		except getopt.GetoptError:
			usage()
			raise SysExitException(2)
		for opt, arg in opts:
			if opt in ['-h', '--help']:
				usage()
				return 0
			elif opt in ['--test']:
				test = True
				test_address = arg
			elif opt in ['--dry']:
				dry = True
			elif opt in ['-d', '--debug']:
				debug += 1
			elif opt in ['-v', '--verbose']:
				verbose = True
			elif opt in ['-c', '--conf']:
				config_file = arg
			elif opt in ['--time']:
				now = datetime.strptime(arg, g.LDAP_TIME_FORMAT)
			elif opt in ['-k']:
				ignore_cert = True
			elif opt in ['--restrict']:
				restrict_to_users = arg
			elif opt in ['--print-conf']:
				print_conf = True

		# no args allow other than options
		if args:
			usage()
			return 2
			
		# set globals
		g.DEBUG = debug
		g.VERBOSE = verbose
		g.NOW = now

		# setup logging
		log.basicConfig(level=log.DEBUG if g.DEBUG else log.INFO if g.VERBOSE else log.WARN,
					format='%(asctime)s %(levelname)s: %(message)s')

		# load configuration
		preconfig = ldap_notify.config.load(filename=config_file)
		
		# merge config with values from command line
		if ignore_cert!=None:      preconfig.set('common', 'ignore_cert', 'true' if ignore_cert else 'false')
		if dry!=None:              preconfig.set('common', 'dry', 'true' if dry else 'false')
		if test!=None:             preconfig.set('test', 'enabled', 'true' if test else 'false')
		if test_address!=None:     preconfig.set('test', 'to_address', test_address)
		if restrict_to_users!=None:preconfig.set('common', 'restrict_to_users', restrict_to_users)

		# evaluate config_file
		config = ldap_notify.config.evaluate(preconfig)
		log.debug('dry = %s, %s' % (str(dry), str(config.dry)))

		# print config only?
		if print_conf:
			import StringIO
			f = StringIO.StringIO()
			preconfig.write(f)
			print f.getvalue()
			return 0
		
		# only proceed if a config_file was given. Especially --print-conf should work without
		if not config_file:
			usage()
			return 2

		# the actual code doing stuff
		run(config)

	except ConfigParser.NoOptionError, e:
		print >> sys.stderr, "Configuration error: %s" % str(e)
		return 2
	except ConfigError, e:
		print >> sys.stderr, "Configuration error: %s" % str(e)
		return 2
	except ldap.LDAPError, e:
		msg = e.args[0]['desc'] if args in e and len(e.args)>=1 and 'desc' in e.args[0] else str(e)
		print >> sys.stderr, "LDAP error: %s" % msg
		return 1
	except SMTPException, e:
		print >> sys.stderr, "SMTP error: %s" % str(e)
		return 1
	except KeyboardInterrupt:
		return 253
	except SysExitException, e:
		return e.rc
	else:
		return 0

# run app in standalone mode
if __name__ == "__main__":
	rc = main(sys.argv[1:])
	sys.exit(rc)



import ldap
from datetime import timedelta, datetime
import logging
log = logging.getLogger('ldap_notify.algorithm')
                    
import ldap_notify.globals as g
import ldap_notify.utils as utils
import ldap_notify.mail as mail

def search_users(config, con, fltr=""):
    attr_list = ["mail", "cn", config.notify_attribute, config.expiry_attribute, 'fullName']
    fltr = "(&(objectClass=%s)(!(loginDisabled=true))%s)" % (config.user_objectclass, fltr)
    users = []
    for dn in (config.base_dn or ['']):
        log.info("Searching for %s at '%s'%s" % (fltr, dn, ' and its subtree' if config.subtree_search else ''))
        dn_users = con.search_s(dn, ldap.SCOPE_SUBTREE if config.subtree_search else ldap.SCOPE_ONELEVEL, fltr, attr_list)
        users.extend(dn_users)
    return users

def ldap_user_to_user(config, cn, ldap_user):
    json_user = {
        'dn': cn,
        'cn': None,
        'mail': None,
        config.notify_attribute: None,
        config.expiry_attribute: None,
        'fullName': None,
        'rule': None
    }
    for attr in ldap_user:
        if attr==config.notify_attribute:
            to_attr = 'notify'
        elif attr==config.expiry_attribute:
            to_attr = 'expiry'
        else:
            to_attr = attr
        json_user[to_attr] = ldap_user[attr][0]
    return utils.obj(json_user)
    

def search_users_without_grace_logins(config, con):
    users = search_users(config, con, "(loginGraceRemaining=0)")    
    return list(ldap_user_to_user(config, cn, ldap_user) for cn, ldap_user in users)


def users_for_rule(config, con, rule):
    users = search_users(config, con, "(&(%s>=%s)(!(%s>=%s))(!(loginGraceRemaining=0)))" % (config.expiry_attribute, rule.start, config.expiry_attribute, rule.end))    
    result = []
    for cn, ldap_user in users:
        # filter out those whose notify attribute shows a notification in less of the days of this rule
        if config.notify_attribute in ldap_user:
            try:
                # check old notify attribute
                if config.notify_attribute in ldap_user:
                    notify_attribute_value = ldap_user[config.notify_attribute][0]
                    parts = notify_attribute_value.split(':')                    
                        
                    def fix_user():
                        log.warn("%sDeleting invalid attribute of %s: '%s: %s'" % ('DRY: ' if config.dry else 'TEST:' if config.test.enabled else '',
                                                                                 cn, config.notify_attribute, notify_attribute_value))
                        if not config.dry:
                            con.modify_s(cn, [
                                (ldap.MOD_DELETE, config.notify_attribute)
                            ])

                    if len(parts) != 2:
                        fix_user()
                    else:
                        # try notify attribute type conversion
                        try:
                            last_rule = int(parts[1])
                            last_notify = datetime.strptime(parts[0], g.LDAP_TIME_FORMAT)
                            expiry = datetime.strptime(ldap_user[config.expiry_attribute][0], g.LDAP_TIME_FORMAT)
                        except TypeError:
                            fix_user()
                        else:                                
                            # skip users without email: their notification is only sent once to the admins.
                            # At this point this happened already.
                            if 'mail' not in ldap_user:
                                log.debug("Skipping %s because reminder %s was sent to admins before: %s" % (cn, last_rule, ldap_user[config.notify_attribute]))
                                continue
                                                    
                            if not (expiry - last_notify >= timedelta(days=last_rule) or last_rule > rule.days):
                                if g.DEBUG > 1:
                                    log.debug("Skipping %s because reminder %s was sent before: %s" % (cn, last_rule, ldap_user[config.notify_attribute]))
                                continue
                    
                # users without email will notify the admin later
                if 'mail' not in ldap_user:
                    log.info("User %s has no email" % cn)
            except ValueError, e:
                log.exception('Skipping %s because: %s' % (cn, str(e)))
                continue
            except Exception, e:
                log.exception(e)
                continue

        log.debug('Found %s with %s=%s to be notified with rule %i' % (cn, config.expiry_attribute, ldap_user[config.expiry_attribute][0], rule.days))
        result.append(ldap_user_to_user(config, cn, ldap_user))

    return result


def mark_user_notified(config, con, user, rule, restricted):
    marker = g.NOW.strftime(g.LDAP_TIME_FORMAT) + ':' + str(rule.days)
    log.info('%sMarking user %s notified with %s' % ('RESTRICTED: ' if restricted else 'DRY: ' if config.dry else '', 
                                                    user.cn, marker))
    if not restricted and not config.dry and not config.test.enabled:
        con.modify_s(user.dn, [
            (ldap.MOD_REPLACE, config.notify_attribute, marker)
        ])
    

def notify_users(config, con, users, rule):
    mailer = mail.MailHandler(config)
    failed = []
    notified = []
    for user in users:
        try:
            restricted = config.restrict_to_users and not (user.cn in config.restrict_to_users or user.dn in config.restrict_to_users)
            mailer.send_user_mail(rule, user, restricted)
            mark_user_notified(config, con, user, rule, restricted)
            notified.append(user)
        except Exception:
            log.exception('Exception processing user %s' + user.cn)
            failed.append(user)
    return notified, failed


def user_to_rule_line(user, rule):
    return "%s, %s%sExpiry Date: %s" % (
        user.dn, 
        (user.mail+', ') if user.mail else '', 
        ("%i Days Rule, " % user.rule.days) if user.rule else '',
        str(datetime.strptime(user.expiry, '%Y%m%d%H%M%SZ')))


def notify_admin(config, con, notified_users, failed_users, users_without_email, users_without_grace_logins):
    mailer = mail.MailHandler(config)
    
    notified_lines = list(user_to_rule_line(user, user.rule) for user in notified_users)
    failed_lines = list(user_to_rule_line(user, user.rule) for user in failed_users)
    without_email_lines = list(user_to_rule_line(user, user.rule) for user in users_without_email)
    no_grace_logins_lines = list(user_to_rule_line(user, user.rule) for user in users_without_grace_logins)
    
    mailer.send_admin_report(config, notified_lines, failed_lines, without_email_lines, no_grace_logins_lines)
            

    



import ldap
import logging
log = logging.getLogger('ldap_notify.connection')

# set timeouts to 10s
ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, 10.0)
ldap.set_option(ldap.OPT_TIMEOUT, 10.0)

# connect to ldap
def connect_to_ldap(config):
    # configure TLS
    starttls = False
    if config.server.startswith('ldaps:') or config.starttls:
        log.info("Using SSL/TLS")
        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)
        starttls = True
    
    if config.ignore_cert:
        log.info("Ignoring SSL/TLS certificate")
        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_ALLOW)
    
    log.info('Connecting to ' + config.server)
    con = ldap.initialize(config.server)
    
    if starttls:
        log.info('Starting TLS')
        con.start_tls_s()
    
    if config.bind_dn:
        log.info('Starting LDAP bind with ' + config.bind_dn)
        con.simple_bind_s(config.bind_dn, config.bind_password)

    return con



from datetime import datetime
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import locale
import logging
log = logging.getLogger('ldap_notify.mail')
import smtplib
from string import Template

import ldap_notify.globals as g
from ldap_notify import ConfigError

def capitalize(line):
    return ' '.join([s[0].upper() + s[1:] for s in line.split(' ')])

class MailHandler(object):
    def __init__(self, config):
        self.config = config
        self.con = None
        self.templates = {}
        
    def smtp_connection(self):
        if self.con:
            return self.con
        
        if not self.config.smtp.server:
            raise ConfigError('No SMTP server defined')
        
        log.debug("Connecting to SMTP server '%s'" % self.config.smtp.server)
        if self.config.smtp.ssl:
            s = smtplib.SMTP_SSL(host=self.config.smtp.server, timeout=30)
        else:
            s = smtplib.SMTP(host=self.config.smtp.server, timeout=30)
        s.connect()
        s.ehlo()
        log.debug("Connected to SMTP server")
        if self.config.smtp.starttls:
            log.debug("Starting TLS")
            s.starttls()
        if self.config.smtp.user:
            log.debug("Authenticating with SMTP server")
            s.login(self.config.smtp.user, self.config.smtp.password)
        log.debug("SMTP connection established")
        self.con = s
        return self.con

    def close_con(self):
        if self.con:
            log.info("Closing Connection to SMTP Server")
            self.con.quit()
            self.con = None
        
    def __del__(self):
        self.close_con()
    
    def template(self, filename):
        if filename in self.templates:
            return self.templates[filename]
        
        f = open(filename, 'r')
        self.templates[filename] = Template(f.read())
        f.close()
        
        return self.templates[filename]

    def send_user_mail(self, rule, user, restricted=False):
        expiry_time = datetime.strptime(user.expiry, '%Y%m%d%H%M%SZ')
        template_env = {
            'expiry_date': expiry_time.date().strftime(locale.nl_langinfo(locale.D_FMT)),
            'days_left': str((expiry_time - g.NOW).days),
            'weeks_left': str((expiry_time - g.NOW).days / 7),
            'months_left': str((expiry_time - g.NOW).days / 30),
            'rule_days': str(rule.days),
            'cn': user.cn,
            'dn': user.dn,
            'fullname': user.fullName if user.fullName else user.cn,
            
            'object': self.config.object,
            'objects': self.config.objects,
            'Object': capitalize(self.config.object),
            'Objects': capitalize(self.config.objects)
        }
        
        # send where?
        to = user.mail if (not self.config.test.enabled) or restricted or self.config.dry else self.config.test.to_address
        
        # send what?
        subject = Template(rule.subject).substitute(template_env)
        from_text = Template(rule.from_text).substitute(template_env) if rule.from_text else None
        verbose_from = (from_text + ' <' + rule.from_address +'>') if from_text else rule.from_address
        
        if rule.text_template:
            text = self.template(rule.text_template).substitute(template_env)
        if rule.html_template:
            # create mime body
            html_msg = MIMEText(self.template(rule.html_template).substitute(template_env), 'html', 'utf-8')
            if rule.text_template:
                text_msg = MIMEText(text, 'plain', 'utf-8')
                mime_msg = MIMEMultipart('alternative')
                mime_msg.attach(html_msg)
                mime_msg.attach(text_msg)
            else:
                mime_msg = html_msg

            # setting mime mail headers
            mime_msg['Subject'] = subject
            mime_msg['From'] = verbose_from
            mime_msg['To'] = to
            
            # convert to string
            msg = mime_msg.as_string()
        elif rule.text_template:
            header = """Content-Type: text/plain; charset="utf-8"
Subject: %s
From: %s
To: %s

""" % (
    subject,
    verbose_from,
    to
)
            msg = header + text
        else:
            raise ConfigError('Neither text_template nor html_template is defined for rule ' + str(rule.days))
        
        # send message
        log.info('%sSending mail for %s to %s: %s' % ('RESTRICTED: ' if restricted else 'DRY: ' if self.config.dry else '', user.cn, to, subject))
        if not restricted and not self.config.dry:
            try:
                smtp_con = self.smtp_connection()
                smtp_con.sendmail(rule.from_address, to, msg)
            except smtplib.SMTPException:
                self.close_con()
                raise
        
    def send_admin_report(self, config, notified_lines, failed_lines, without_email_lines, no_grace_logins_lines):
        # send where?
        to = config.admin.to_address if not self.config.test.enabled else self.config.test.to_address
        
        # send what
        template_env = {
            'notified_users': '\n'.join(notified_lines),
            'failed_users': '\n'.join(failed_lines),
            'users_without_email': '\n'.join(without_email_lines),
            'no_grace_logins': '\n'.join(no_grace_logins_lines),
            
            'notified_users_length': len(notified_lines),
            'failed_users_length': len(failed_lines),
            'users_without_email_length': len(without_email_lines),
            'no_grace_logins_length': len(no_grace_logins_lines),

            'object': self.config.object,
            'objects': self.config.objects,
            'Object': capitalize(self.config.object),
            'Objects': capitalize(self.config.objects),
        }
        admin_template = self.template(config.admin.text_template)
        msg = admin_template.substitute(template_env)
        
        # setting mail headers
        subject = Template(config.admin.subject).substitute(template_env)
        from_text = Template(config.admin.from_text).substitute(template_env) if config.admin.from_text else None
        header = """Content-Type: text/plain; charset="utf-8"
Subject: %s
From: %s
To: %s

""" % (
    subject,
    (from_text + ' <' + config.admin.from_address +'>') if from_text else config.admin.from_address,
    to
)

        # send message
        log.info('%sSending admin report to %s: %s' % ('DRY: ' if self.config.dry else '', to, subject))
        if self.config.dry:
            print
            print str(header+msg)
        else:
            try:
                smtp_con = self.smtp_connection()
                smtp_con.sendmail(config.admin.from_address, to, header+msg)
            except smtplib.SMTPException:
                self.close_con()
                raise



#!/usr/bin/env python

from setuptools import setup

setup(name='ldap-notify',
	description='LDAP Login/Password Email Notification Service',
	version='0.8.3',
	long_description="""\
      `ldap-notify` searches users in an LDAP directory whose login or password expires in a certain number of days. These users are notified with a customizable email. The time intervals for the notifications and the email templates can be freely defined. By using a custom LDAP attribute the last notification is stored in the directory such that users are only notified once per rule. Moreover, this mechanism allows running the tool (e.g. via cron) on multiple machine with a certain time delta in order to implement high availabiliy. 
      """,
    author='Dr. Stefan Schimanski',
    author_email='stefan.schimanski@gmail.com',
    url='https://github.com/sttts/ldap_notify',
    packages=['ldap_notify'],
    package_data = {'ldap_notify': ['templates/*']},
    scripts=[ 'scripts/ldap-notify' ],
	classifiers=[
		"License :: OSI Approved :: GNU General Public License (GPL)",
	  	"Programming Language :: Python",
	  	"Development Status :: 4 - Beta",
	  	"Topic :: System :: Systems Administration :: Authentication/Directory :: LDAP"
	],
	keywords='ldap notification eDirectory',
	license='GPL',
	install_requires=[
		'python-ldap',
	]
)



import os
import socket
from StringIO import StringIO
import logging
log = logging.getLogger('ldap-notify.config')

from ldap_notify import ConfigError
import ldap_notify.utils as utils
import ldap_notify.globals as g

HOSTNAME = socket.gethostname()

# set defaults
default_cfg = """\
[common]
server = ldap://localhost
bind_dn =
bind_password =
bind_password_base64 =
starttls = false
ignore_cert = false
base_dn =
subtree_search = false 
expiry_attribute = passwordExpirationTime
notify_attribute = pwmNotify
dry = false
restrict_to_users =
user_objectclass = person
object = password
objects =

[smtp]
server =
ssl = false
starttls = false
user =
password =
password_base64 =

[admin]
from_address = {root_mail!s}
to_address = {admin_mail!s}
from_text = $Object Expiry Notification
subject = $Object Expiry Notification Report
text_template = {text_template!s}

[test]
enabled = false
to_address = {root_mail!s}
""".format(
    root_mail="root@" + HOSTNAME,
    admin_mail="admin@" + HOSTNAME,
    text_template=os.path.dirname(__file__) + "/templates/admin.tmpl.txt"
)

def flatten(l):
    return list(x for sublist in l for x in sublist)

def dn_list_parse(s):
    lst = s.replace(';', '\n').replace(' ', '\n').split('\n')
    return filter(None, lst)

# load config file
def load(filename = None):
    import ConfigParser
    config_file = ConfigParser.SafeConfigParser()
    config_file.readfp(StringIO(default_cfg))
    if filename:
        log.info('Reading config file %s' % filename)
        config_file.readfp(open(filename))
    
    return config_file

def evaluate(config):
    c = {}
    
    c['server'] = config.get("common", "server").strip() or None
    c['bind_dn'] = config.get("common", "bind_dn").strip() or None
    c['bind_password'] = config.get("common", "bind_password").strip() or None
    c['bind_password_base64'] = config.get("common", "bind_password_base64").strip() or None
    c['starttls'] = config.getboolean("common", "starttls")
    c['ignore_cert'] = config.getboolean("common", "ignore_cert")
    c['base_dn'] = dn_list_parse(config.get("common", "base_dn").strip())
    c['subtree_search'] = config.getboolean("common", "subtree_search") or c['base_dn'] == []
    c['expiry_attribute'] = config.get("common", "expiry_attribute").strip() or None
    c['notify_attribute'] = config.get("common", "notify_attribute").strip() or None
    c['user_objectclass'] = config.get("common", "user_objectclass").strip() or None
    c['dry'] = config.getboolean("common", "dry")
    c['restrict_to_users'] = set(dn_list_parse(config.get("common", "restrict_to_users", "").strip()))
    c["object"] = config.get("common", "object").strip()
    c["objects"] = config.get("common", "objects").strip() or (c["object"] + 's')
    
    c['smtp'] = {}
    c['smtp']['server'] = config.get("smtp", "server").strip() or None
    c['smtp']['ssl'] = config.getboolean("smtp", "ssl")
    c['smtp']['starttls'] = config.getboolean("smtp", "starttls")
    c['smtp']['user'] = config.get("smtp", "user").strip() or None
    c['smtp']['password'] = config.get("smtp", "password").strip() or None
    c['smtp']['password_base64'] = config.get("smtp", "password_base64").strip() or None
    
    c['admin'] = {}
    c['admin']['from_address'] = config.get("admin", "from_address").strip() or None
    c['admin']['to_address'] = config.get("admin", "to_address").strip() or None
    c['admin']['from_text'] = config.get("admin", "from_text").strip() or None
    c['admin']['subject'] = config.get("admin", "subject").strip() or None
    c['admin']['text_template'] = config.get("admin", "text_template").strip() or None
    
    c['test'] = {}
    c['test']['enabled'] = config.getboolean("test", "enabled")
    c['test']['to_address'] = config.get("test", "to_address").strip() or None
    
    c['rules'] = []
    for section in config.sections():
        try:
            rule_num = int(section)
        except ValueError:
            continue

        c['rules'].append({
            "days": rule_num,
            "from_address": config.get(section, "from_address", "root@" + HOSTNAME).strip() if config.has_option(section, "from_address") else c['admin']['from_address'],
            "from_text": config.get(section, "from_text").strip() if config.has_option(section, "from_text") else c['admin']['from_text'],
            "subject": config.get(section, "subject").strip() if config.has_option(section, "subject") else "$Object will expire soon",
            "text_template": config.get(section, "text_template").strip() if config.has_option(section, "text_template") else os.path.dirname(__file__) + "/templates/notify.tmpl.txt",
            "html_template": config.get(section, "html_template").strip() if config.has_option(section, "html_template") else None
        })

    c['rules'] = sorted(c['rules'], key=lambda r: r['days'])
    
    # decode base64 passwords, if set
    import base64
    if c['bind_password'] and c['bind_password_base64']:
        raise ConfigError('Cannot set bind_password and bind_password_base64')
    if not c['bind_password'] and c['bind_password_base64']:
        c['bind_password'] = base64.b64decode(c['bind_password_base64'])

    if c['smtp']['password'] and c['smtp']['password_base64']:
        raise ConfigError('Cannot set SMTP password and password_base64')
    if not c['smtp']['password'] and c['smtp']['password_base64']:
        c['smtp']['password'] = base64.b64decode(c['smtp']['password_base64'])
    
    # compute time interval [start:end[ of each rule
    from datetime import timedelta
    start = g.NOW
    for rule in c['rules']:        
        rule['start'] = start.strftime(g.LDAP_TIME_FORMAT)
        end = g.NOW + timedelta(days=rule['days'])
        rule['end'] = end.strftime(g.LDAP_TIME_FORMAT)
        
        start = end
        
    log.debug(repr(c['rules']))

    return utils.obj(c)
